import logging

from AccessControl import Unauthorized
from Acquisition import aq_inner
from mimetypes import guess_type
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as PMF
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from plone import api
from plone.memoize.view import memoize
from plone.protect.interfaces import IDisableCSRFProtection
from zope.cachedescriptors.property import Lazy
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.lifecycleevent import modified

from Products.Poi import PoiMessageFactory as _
from Products.Poi import permissions
from Products.Poi.adapters import IResponseContainer
from Products.Poi.adapters import Response
from Products.Poi.browser.interfaces import IResponseAdder
from Products.Poi.browser.response import Create as OrigCreate
from Products.Poi.browser.response import AddForm as OrigAdd
from Products.Poi.browser.response import voc2dict
from Products.Poi.content.tracker import possibleAssignees
from Products.Poi.content.tracker import possibleSeverities
from Products.Poi.content.tracker import possibleTargetReleases


class Create(OrigCreate):

    def __call__(self):
        form = self.request.form
        context = aq_inner(self.context)
        request = context.REQUEST
        authenticator = getMultiAdapter((context, request),
                                        name=u"authenticator")
        # CSRF should be disabled during tests
        if (not IDisableCSRFProtection.providedBy(request) and
           not authenticator.verify()):
            raise Unauthorized
        if not self.memship.checkPermission('Poi: Add Response', context):
            raise Unauthorized

        response_text = form.get('response', u'')

        software_engineer_time = form.get('software_engineer_time', 0)
        if software_engineer_time:
            if not context.software_engineer_time_total:
                context.software_engineer_time_total = 0
            context.software_engineer_time_total += \
                float(software_engineer_time)
            response_text += "\n<p>Software Engineer: " + \
                software_engineer_time + " Hours</p>"

        business_analyst_time = form.get('business_analyst_time', 0)
        if business_analyst_time:
            if not context.business_analyst_time_total:
                context.business_analyst_time_total = 0
            context.business_analyst_time_total += \
                float(business_analyst_time)
            response_text += "\n<p>Business Analyst: " + \
                business_analyst_time + " Hours</p>"

        transend_edi_support_time = form.get('transend_edi_support_time', 0)
        if transend_edi_support_time:
            if not context.transend_edi_support_time_total:
                context.transend_edi_support_time_total = 0
            context.transend_edi_support_time_total += \
                float(transend_edi_support_time)
            response_text += "\n<p>TranSend/EDI Support: " + \
                transend_edi_support_time + " Hours</p>"

        system_engineer_time = form.get('system_engineer_time', 0)
        if system_engineer_time:
            if not context.system_engineer_time_total:
                context.system_engineer_time_total = 0
            context.system_engineer_time_total += \
                float(system_engineer_time)
            response_text += "\n<p>System Engineer: " + \
                system_engineer_time + " Hours</p>"

        operations_roi_time = form.get('operations_roi_time', 0)
        if operations_roi_time:
            context.operations_roi_time = float(operations_roi_time)
            response_text += "\n<p>Operations ROI: " + \
                operations_roi_time + " Hours</p>"

        new_response = Response(response_text)
        new_response.mimetype = self.mimetype
        new_response.type = self.determine_response_type(new_response)

        issue_has_changed = False
        transition = form.get('transition', u'')
        if transition and transition in self.available_transitions:
            wftool = getToolByName(context, 'portal_workflow')
            before = wftool.getInfoFor(context, 'review_state')
            before = wftool.getTitleForStateOnType(before, 'PoiIssue')
            wftool.doActionFor(context, transition)
            after = wftool.getInfoFor(context, 'review_state')
            after = wftool.getTitleForStateOnType(after, 'PoiIssue')
            new_response.add_change('review_state', _(u'Issue state'),
                                    before, after)
            issue_has_changed = True

        options = [
            ('severity', _(u'Severity'), 'available_severities'),
            ('current_assignee', _(u'Assignee'),
             'available_assignees'),
            ('targetRelease', _(u'Target release'), 'available_releases'),
        ]
        for option, title, vocab in options:
            new = form.get(option, u'')
            if new and new in self.__getattribute__(vocab):
                current = self.__getattribute__(option)
                if current == new:
                    continue
                new_response.add_change(option, title, current, new)
                issue_has_changed = True
                if option == 'severity':
                    context.severity = new
                elif option == 'targetRelease':
                    context.target_release = new
                elif option == 'current_assignee':
                    context.assignee = new

        if issue_has_changed:
            context.reindexObject()

        if len(response_text) == 0 and not issue_has_changed:
            status = IStatusMessage(self.request)
            msg = _(u"No response text added and no issue changes made.")
            msg = translate(msg, 'Poi', context=self.request)
            status.addStatusMessage(msg, type='error')
        else:
            # Add response
            self.folder.add(new_response)
        redirect_url = "{0}?_authenticator={1}".format(context.absolute_url(),
                                                       authenticator.token())
        self.request.response.redirect(redirect_url)


class AddForm(OrigAdd):

    def get_key(self, elem):
        return elem['label']

    @property
    def assignees_for_display(self):
        """Get the tracker assignees as a dictionary.
        """

        assignees = voc2dict(self.available_assignees, self.current_assignee)
        return sorted(assignees, key=self.get_key);
