# -*- coding: utf-8 -*-
from zope import schema
from zope.component import provideAdapter
from zope.interface import implementer, Invalid

from z3c.form import validator

from collective import dexteritytextindexer
from plone.app.z3cform.widget import AjaxSelectFieldWidget, DateFieldWidget
from plone.autoform.directives import \
    order_after, \
    omitted, \
    widget, \
    write_permission, \
    read_permission

from Products.Poi import PoiMessageFactory as _
from Products.Poi.content.tracker import possibleAssignees
from Products.Poi.content.issue import IIssue as IIssueOrig
from Products.Poi.content.issue import Issue as IssueOrig

from sunpower.poi.content.sunpowertracker import \
    possibleOrigins, \
    possibleCarriers, \
    possibleTypesOfIssues, \
    possibleClaimStatuses, \
    possibleSubCategories, \
    possibleWhy1Options, \
    possibleWhy2Options


class ISunPowerIssue(IIssueOrig):
    """Customization of marker interface for Poi issue"""

    dexteritytextindexer.searchable('type_of_issue')
    type_of_issue = schema.Choice(
        title=_(u'Poi_label_type_of_issue', default=u'Category'),
        description=_(u'Poi_help_type_of_issue',
                      default=u"Select the type of issue."),
        source=possibleTypesOfIssues,
        required=True,
    )

    dexteritytextindexer.searchable('type_of_issue')
    sub_category = schema.Choice(
        title=_(u'Poi_label_sub_category', default=u'Sub-Category'),
        description=_(u'Poi_help_sub_category',
                      default=u"Select the sub-category of the issue."),
        source=possibleSubCategories,
        required=True,
    )

    requested_by = schema.Date(
        title=_(u'label_requestedby_date', u'Response Due Date'),
        description=_(
            u'help_requestedby_date',
            default=u"Select the date when the issue must be completed."),
        required=False
    )

    dexteritytextindexer.searchable('delivery_number')
    delivery_number = schema.TextLine(
        title=_(u'Poi_label_delivery_number', default=u'Delivery #'),
        required=True,
    )

    dexteritytextindexer.searchable('load_id')
    load_id = schema.TextLine(
        title=_(u'Poi_label_load_id', default=u'Load ID'),
        required=False,
    )

    dexteritytextindexer.searchable('origin_name')
    origin_name = schema.Choice(
        title=_(u'Poi_label_origin_name', default=u'Origin Name'),
        source=possibleOrigins,
        required=True,
    )

    dexteritytextindexer.searchable('origin_city')
    origin_city = schema.TextLine(
        title=_(u'Poi_label_origin_city', default=u'Origin City'),
        required=False,
    )

    dexteritytextindexer.searchable('origin_state')
    origin_state = schema.TextLine(
        title=_(u'Poi_label_origin_state', default=u'Origin State'),
        required=False,
    )

    dexteritytextindexer.searchable('origin_country')
    origin_country = schema.TextLine(
        title=_(u'Poi_label_origin_country', default=u'Origin Country'),
        required=False,
    )

    read_permission(destination_name='cmf.AddPortalContent')
    write_permission(destination_name='cmf.AddPortalContent')
    widget('destination_name',
           AjaxSelectFieldWidget,
           vocabulary='sunpower.poi.DestinationName',
           pattern_options={
               'allowNewItems': 'true'
           })
    destination_name = schema.Tuple(
        title=_(u'Poi_label_destination_name', default=u'Destination Name'),
        value_type=schema.TextLine(),
        required=True,
    )

    dexteritytextindexer.searchable('destination_city')
    destination_city = schema.TextLine(
        title=_(u'Poi_label_destination_city', default=u'Destination City'),
        required=False,
    )

    dexteritytextindexer.searchable('destination_state')
    destination_state = schema.TextLine(
        title=_(u'Poi_label_destination_state', default=u'Destination State'),
        required=False,
    )

    dexteritytextindexer.searchable('carrier')
    carrier = schema.Choice(
        title=_(u'Poi_label_carrier', default=u'Carrier'),
        source=possibleCarriers,
        required=False,
    )

    ship_date = schema.Date(
        title=_(u'label_ship_date', u'Ship Date'),
        required=False
    )

    assignee = schema.Choice(
        title=_(u'Poi_label_issue_assignee', default=u'Responsible Manager'),
        description=_(u'Poi_help_issue_assignee',
                      default=u"Select the user responsible "
                      u"for issue management."),
        source=possibleAssignees,
        required=False,
    )

    dexteritytextindexer.searchable('claim_status')
    claim_status = schema.Choice(
        title=_(u'Poi_label_claim_status', default=u'Claim Status'),
        source=possibleClaimStatuses,
        required=False,
    )

    dexteritytextindexer.searchable('why1')
    why1 = schema.Choice(
        title=_(u'Poi_label_why1', default=u'Why - 1'),
        source=possibleWhy1Options,
        required=False,
    )

    dexteritytextindexer.searchable('why2')
    why2 = schema.Choice(
        title=_(u'Poi_label_why2', default=u'Why - 2'),
        source=possibleWhy2Options,
        required=False,
    )

    dexteritytextindexer.searchable('why3')
    why3 = schema.Text(
        title=_(u'Poi_label_why3', default=u'Why - 3'),
        required=False,
    )

    dexteritytextindexer.searchable('preventative_measure')
    preventative_measure = schema.Text(
        title=_(u'Poi_label_preventative_measure',
                default=u'Preventative Measure'),
        required=False,
    )

    preventative_due = schema.Date(
        title=_(u'label_preventative_due', u'Preventative Measure Due Date'),
        required=False
    )
    widget('preventative_due', DateFieldWidget)


    # remove unused fields
    omitted('release',
            'steps',
            'area',
            'issue_type',
            'severity',
            'target_release',
            'watchers',
            )

    # order things the way we want them
    order_after(related_issue='assignee')
    order_after(subject='assignee')
    order_after(contact_email='ship_date')


@implementer(ISunPowerIssue)
class SunPowerIssue(IssueOrig):
    """
    An issue in the Poi Tracker
    """

    DISPLAY_OPTION_FIELDS = {
        'possibleOrigins': (possibleOrigins, 'origin_name'),
        'possibleCarriers': (possibleCarriers, 'carrier'),
        'possibleTypesOfIssues': (possibleTypesOfIssues, 'type_of_issue'),
        'possibleClaimStatuses': (possibleClaimStatuses, 'claim_status'),
        'possibleSubCategories': (possibleSubCategories, 'sub_category'),
        'possibleWhy1Options': (possibleWhy1Options, 'why1'),
        'possibleWhy2Options': (possibleWhy2Options, 'why2')}

    def display_option(self, key, tracker=None):
        """
        Lookup the vocabulary term title from the tracker if available.
        """
        vocab_factory, attr = self.DISPLAY_OPTION_FIELDS[key]
        value = getattr(self, attr)

        if tracker is None:
            tracker = self.getTracker()
        vocab = vocab_factory(tracker)
        try:
            value = vocab.by_value[value].title
        except KeyError:
            pass

        return value


class EmptyValidation(validator.SimpleFieldValidator):
    """check for custom empty0 value on choice fields"""
    def validate(self, value):
        if 'empty0' in value:
            raise Invalid(_(u"You must make a selection for this field."))


class EmptyValidation2(EmptyValidation):
    """check for custom empty0 value on choice fields"""


class EmptyValidation3(EmptyValidation):
    """check for custom empty0 value on choice fields"""


# can't use a validator multiple times, or I don't know how
# see https://github.com/zopefoundation/z3c.form/issues/61
validator.WidgetValidatorDiscriminators(
    EmptyValidation, field=ISunPowerIssue['origin_name'])
validator.WidgetValidatorDiscriminators(
    EmptyValidation2, field=ISunPowerIssue['type_of_issue'])
validator.WidgetValidatorDiscriminators(
    EmptyValidation3, field=ISunPowerIssue['sub_category'])
provideAdapter(EmptyValidation)
provideAdapter(EmptyValidation2)
provideAdapter(EmptyValidation3)
